<?php

namespace App\Http\Controllers\V1\Canje;

use App\Dto\CanjeDto;
use App\Http\Controllers\Controller;
use App\Services\CanjeService;
use App\Services\EmailService;
use App\Services\PlayerService;
use App\Services\RecompensaService;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;

class CanjeController extends Controller
{

    protected $canjeService;
    protected $recompensaService;
    protected $playerService;
    protected $emailService;

    public function __construct(CanjeService $canjeService, RecompensaService $recompensaService, PlayerService $playerService, EmailService $emailService)
    {
        $this->canjeService = $canjeService;
        $this->recompensaService =  $recompensaService;
        $this->playerService = $playerService;
        $this->emailService = $emailService;
    }

    /**
     * The function `getRecompensasCanje` retrieves all rewards for redemption and handles error cases.
     * 
     * @return The function `getRecompensasCanje` is returning the data from the ``
     * variable if the query to get all canjes is successful. If the query is not successful, it
     * returns a JSON response with an error message indicating "Error al consultar los canjes" and a
     * status code of 400.
     */
    public function getRecompensasCanje()
    {
        $resultQuery = $this->canjeService->getCanjesAll();
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los canjes"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * This PHP function retrieves rewards for redemption by a participant based on their ID.
     * 
     * @param idParticipante The `getRecompensasCanjeByParticipante` function takes in a parameter
     * ``, which is the ID of a participant. This function retrieves the rewards that
     * can be redeemed by the participant with the given ID. It first calls the `getCanjesByPlayer
     * 
     * @return The function `getRecompensasCanjeByParticipante` returns the data of canjes (rewards)
     * for a specific participant identified by their ID. If the query to retrieve the canjes is
     * successful, it returns the data. If there is an error in the query, it returns a JSON response
     * with an error message indicating the failure to retrieve the canjes.
     */
    public function getRecompensasCanjeByParticipante($idParticipante)
    {
        $resultQuery = $this->canjeService->getCanjesByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus canjes"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * This PHP function retrieves the redemption points for a participant and returns the data or an
     * error message if the query is unsuccessful.
     * 
     * @param idParticipante The `idParticipante` parameter is the identifier of the participant for
     * whom we want to retrieve the points that can be redeemed. This function
     * `getPuntosCanjeParticipante` takes this participant ID as input and then calls the
     * `getPointsByPlayer` method from the `can
     * 
     * @return The function `getPuntosCanjeParticipante` is returning the data obtained from the
     * `getPointsByPlayer` method of the `canjeService`. If the query is successful, it returns the
     * data. If the query is not successful, it returns a JSON response with an error message
     * indicating "Error al consultar tus puntos de canje" and a status code of 400.
     */
    public function getPuntosCanjeParticipante($idParticipante)
    {
        $resultQuery = $this->canjeService->getPointsCanjeByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus puntos de canje"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function `createCanjeParticipante` processes a request to create a reward exchange for a
     * participant, validating the input data and handling various error scenarios.
     * 
     * @param Request request The `createCanjeParticipante` function is responsible for creating a new
     * "canje" (exchange) for a participant. Let me explain the process step by step:
     * 
     * @return a JSON response with different messages based on the outcome of the process. Here are
     * the possible return scenarios:
     */
    public function createCanjeParticipante(Request $request)
    {
        $data = $request->only('participante_id', 'recompensa_id');
        $validator = Validator::make($data, [
            'participante_id' => 'integer|required|exists:participantes,id',
            'recompensa_id' => 'integer|required|exists:recompensas,id',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $idParticipante = $request->participante_id;
            $resultFindRecompensa = $this->recompensaService->getFindRecompensa($request->recompensa_id);
            if (!$resultFindRecompensa->success) {
                return response()->json(['error' => ['mensaje' => ["Error al buscar la recompensa"]]], 400);
            }
            $recompensaCanje = $resultFindRecompensa->recompensa;
            //puntos meta canje
            $puntosCaje = $recompensaCanje->metaCanje;
            //obtengo los puntos de canje disponibles para usar del participante
            $resultPointCanjePlayer = $this->canjeService->getPointsCanjeByPlayer($idParticipante);
            if (!$resultPointCanjePlayer->success) {
                return response()->json(['error' => ['mensaje' => ["Error al obtener tus puntos de canje válidos."]]], 400);
            }
            //puntos disponibles canje
            $totalPuntosDisponibleCanje = $resultPointCanjePlayer->data;
            if (!($totalPuntosDisponibleCanje >= $puntosCaje)) {
                return response()->json(['error' => ['mensaje' => ["No tienes suficientes puntos para canjear la recompensa"]]], 400);
            }

            //creo el canje si es que tiene los puntos válidos
            $canjeDto = new CanjeDto();
            $canjeDto->participanteId = $request->participante_id;
            $canjeDto->recompensaId = $request->recompensa_id;
            $canjeDto->puntosCanjeados = $puntosCaje;
            $resultCreateRecompensa = $this->canjeService->addCanje($canjeDto);
            if(!$resultCreateRecompensa->success){
                return response()->json(['error' => ['mensaje' => ["Error al crear el canje"]]], 400);
            }
            return response()->json([
                'message' => 'El canje se creo con éxito.',
                'data' => $resultCreateRecompensa->canje
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al crear canje: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al cajear la recompensa"]]], 400);
        }
    }

    /**
     * The function `changeStatusCanje` updates the status of a redemption process and sends an email
     * notification to the participant if the status is approved.
     * 
     * @param Request request The `changeStatusCanje` function is responsible for updating the status
     * of a "canje" (exchange) based on the provided request data. Let me explain the flow of this
     * function:
     * 
     * @return a JSON response with different possible outcomes:
     */
    public function changeStatusCanje(Request $request)
    {
        $data = $request->only('status', 'canje_id');
        $validator = Validator::make($data, [
            'status' => 'string|required|in:aprobado,rechazado',
            'canje_id' => 'integer|required|exists:canjes,id',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            DB::beginTransaction();
            $canjeDto = new CanjeDto();
            $canjeDto->procesoEstatus = $request->status;
            $resultUpdateCanje = $this->canjeService->updateStatusProcessCanje($canjeDto, $request->canje_id);
            if(!$resultUpdateCanje->success){
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al actualizar el estatus del canje"]]], 400);
            }
            $idParticipante = $resultUpdateCanje->canje->participanteId;
            $idRecompensa = $resultUpdateCanje->canje->recompensaId;
            $numCanje = $resultUpdateCanje->canje->numeroCanje;
            $puntosCanjeados = $resultUpdateCanje->canje->puntosCanjeados;

            $resultParticipanteGet = $this->playerService->getPlayerByIdWithUserOnlyData($idParticipante);
            if(!$resultParticipanteGet->success){
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al obtener información del participante para notificar"]]], 400);
            }
            $participante = $resultParticipanteGet->data;
            $resultFindRecompensa = $this->recompensaService->getFindRecompensa($idRecompensa);
            if (!$resultFindRecompensa->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al obtener información de la recompensa para notificar"]]], 400);
            }
            $recompensaCanje = $resultFindRecompensa->recompensa;

            if ($request->status == "aprobado") {
                $details = [
                    'email' => $participante->usuario['email'],
                    'nombre_participante' => $participante->nombre_completo,
                    'num_canje' => $numCanje,
                    'nombre_recompensa' => $recompensaCanje->nombre,
                    'num_puntos_canjeados' => $puntosCanjeados
                ];
                $sendEmailInvitacion = $this->emailService->SendEmailCanjeApprove($details);
                if (!$sendEmailInvitacion) {
                    DB::rollback();
                    return response()->json(['error' => ['mensaje' => ["Error al notificar al participante"]]], 400);
                }
            }
            DB::commit();
            return response()->json([
                'message' => 'Se actualiazó correctamente el estatus del canje',
                'data' => $resultUpdateCanje->canje
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el status del canje"]]], 400);
        }
    }

    /**
     * The function `getCanjeDetalle` retrieves the details of a specific canje and returns the data if
     * successful, otherwise it returns an error message.
     * 
     * @param idCanje The `idCanje` parameter is used as an input to retrieve details of a specific
     * canje (exchange) from the `canjeService`. It is passed to the `getCanjeDetail` method of the
     * `canjeService` to fetch the details of the canje with the provided
     * 
     * @return The function `getCanjeDetalle` is returning the data of the canje detail if the query is
     * successful. If the query is not successful, it returns a JSON response with an error message
     * indicating "Error al obtener el detalle del canje" and a status code of 400.
     */
    public function getCanjeDetalle($idCanje)
    {
        $resultQuery = $this->canjeService->getCanjeDetail($idCanje);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al obtener el detalle del canje"]]], 400);
        }
        return $resultQuery->data;
    }
}
