<?php

namespace App\Http\Controllers\V1\File;

use App\Dto\FileDto;
use App\Http\Controllers\Controller;
use App\Models\File;
use App\Models\User;
use App\Services\AuthService;
use App\Services\FileService;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Storage;

class FileController extends Controller
{
    protected $fileService;
    protected $authService;

    public function __construct(FileService $fileService, AuthService $authService)
    {
        $this->fileService = $fileService;
        $this->authService = $authService;
    }

    public function index($idUser)
    {
        $resultQuery = $this->fileService->getFilesByUser($idUser);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los archivos"]]], 400);
        }
        return $resultQuery->data;
    }


    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('file', 'user_id');
        $validator = Validator::make($data, [
            'file' => 'required|mimes:jpg,jpeg,png,gif,zip,pdf|max:5048',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            if (($request->file('file'))) {
                DB::beginTransaction();
                $name = $request->file('file')->getClientOriginalName();
                $nameSimple = pathinfo($name, PATHINFO_FILENAME);
                $extension = $request->file('file')->getClientOriginalExtension();
                $size = $request->file('file')->getSize();
                $url = '';
                $userId = $request->user_id;
                $contadorResult = $this->fileService->getCountFilesWithRepeatName($nameSimple, $extension, $userId);
                $contadorFiles = $contadorResult->data;
                $nameFile = ($contadorFiles > 0) ? $nameSimple . '-' . $contadorFiles . '.' . $extension : $name;
                $userFind = $this->authService->getUserById($request->user_id);
                if (!$userFind) {
                    DB::rollback();
                    return response()->json(['error' => ['mensaje' => ["Error al procesar el archivo"]]], 400);
                }
                $correoUser = strtolower($userFind->email);
                $path = $request->file('file')->store($correoUser . '/files');
                $fileDto = new FileDto();
                $fileDto->nombre = $nameFile;
                $fileDto->path = $path;
                $fileDto->type = $extension;
                $fileDto->size = $size;
                $fileDto->userId =  $userId;
                $resultSave = $this->fileService->saveFile($fileDto);
                if (!$resultSave->success) {
                    DB::rollback();
                    return response()->json(['error' => ['mensaje' => ["Error al guardar el archivo"]]], 400);
                }

                $url = Storage::url($path);
                $archivo = $resultSave->archivo;
                $idArchivo = $archivo->id;

                DB::commit();
                return response()->json([
                    'id_file' => $idArchivo,
                    'url' => $url
                ], Response::HTTP_OK);
            }
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Error en el controller al guardar el archivo: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al guardar el archivo"]]], 400);
        }
    }

    public function destroy($id)
    {
        try {
            $archivo = File::find($id);
            if (!$archivo) {
                return response()->json(['error' => ['mensaje' => ["El archivo no existe"]]], 400);
            }
            if (Storage::exists($archivo->path)) {
                Storage::delete($archivo->path);
            }
            $resultDelete = $this->fileService->removeFile($id);
            if (!$resultDelete) {
                return response()->json(['error' => ['mensaje' => ["Error al eliminar el archivo"]]], 400);
            }
            return response()->json([
                'message' => 'Se elimino el archivo correctamente'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al eliminar el archivo: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al procesar la eliminación del archivo"]]], 400);
        }
    }
}
