<?php

namespace App\Http\Controllers\V1\Participante;

use App\Dto\PlayerDto;
use App\Dto\UserDto;
use App\Http\Controllers\Controller;
use App\Http\Requests\CreatePlayerRequest;
use App\Http\Requests\UpdatePlayerRequest;
use App\Models\Participante;
use App\Services\AuthService;
use App\Services\EmailService;
use App\Services\PlayerService;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Str;
use Spatie\Permission\Models\Role;

class ParticipanteController extends Controller
{
    protected $playerService;
    protected $authService;
    protected $emailService;

    public function __construct(PlayerService $playerService, AuthService $authService, EmailService $emailService)
    {
        $this->playerService = $playerService;
        $this->authService = $authService;
        $this->emailService = $emailService;
    }

    /**
     * The function retrieves all players and returns them as JSON data, handling errors if the
     * retrieval is unsuccessful.
     * 
     * @return the data obtained from the `getPlayersAll()` method of the ``. If the
     * result is successful, it returns the data. If there is an error, it returns a JSON response with
     * an error message indicating "Error al consultar los participantes" and a status code of 400.
     */
    public function index()
    {
        $result = $this->playerService->getPlayersAll();
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los participantes"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function `getParticipanteRole` retrieves players by role and returns the data or an error
     * message if unsuccessful.
     * 
     * @param role The `getParticipanteRole` function takes a role as a parameter. This role is used to
     * retrieve players by their role using the `getPlayersByRole` method from the `playerService`. If
     * the operation is successful, it returns the data containing the players with the specified role.
     * If
     * 
     * @return The function `getParticipanteRole` is returning the data of players with the specified
     * role obtained from the `playerService`. If the retrieval is successful, it returns the player
     * data. If there is an error during the retrieval process, it returns a JSON response indicating
     * the error message "Error al consultar los participantes por rol" with a status code of 400.
     */
    public function getParticipanteRole($role)
    {
        $result = $this->playerService->getPlayersByRole($role);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los participantes por rol"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function "show" retrieves player details by ID and returns the data or an error message if
     * unsuccessful.
     * 
     * @param id The `show` function takes an `` parameter which is used to retrieve a player's
     * details by calling the `getPlayerById` method from the `playerService`. If the retrieval is
     * successful, it returns the player's data. If there is an error during the retrieval process, it
     * returns a
     * 
     * @return If the `` object's `success` property is false, the function will return a JSON
     * response with an error message. Otherwise, it will return the data from the `` object.
     */
    public function show($id)
    {
        $result = $this->playerService->getPlayerById($id);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el detalle del participante"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function `store` creates a new player, registers a user with a random password, sends an
     * email invitation, and returns a JSON response with the result.
     * 
     * @param CreatePlayerRequest request The `store` function you provided seems to be handling the
     * creation of a new player in your application. Let's break down the parameters that are being
     * passed in the `CreatePlayerRequest`:
     * 
     * @return a JSON response with a success message if the participant is created successfully along
     * with the participant data. If any error occurs during the process (such as validation failure,
     * user creation failure, player creation failure, or email notification failure), it returns a
     * JSON response with an error message. In case of any error, the function also rolls back the
     * database transaction.
     */
    public function store(CreatePlayerRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        DB::beginTransaction();
        $nombreTienda = $request->nombre_tienda ?? "";
        $direccion = $request->direccion ?? "";
        $nombreCompleto = $request->nombre_completo ?? "";
        $localidad = $request->localidad ?? "";
        $estado = $request->estado ?? "";
        $ciudad = $request->ciudad ?? "";
        $ruta = $request->ruta ?? "";
        $fileId = $request->file_id ?? "";
        $identificador = $request->identificador ?? "";

        $randomPassword = Str::random(12);
        $newUserDto = new UserDto();
        $newUserDto->name = $request->nombre_completo ?? "";
        $newUserDto->email = $request->email;
        $newUserDto->password = $randomPassword;
        $resultCreateUser = $this->authService->registerUserRol($newUserDto, $request->role);

        if (!$resultCreateUser->success) {
            DB::rollback();
            return response()->json(['error' => ['mensaje' => ["Error al crear los accesos del participante"]]], 400);
        }

        $userId = $resultCreateUser->user->id;

        $playerDto = new PlayerDto($nombreCompleto, $nombreTienda, $direccion, $localidad, $estado, $ciudad, $ruta, $fileId, $userId, $identificador);
        $resultCreatePlayer = $this->playerService->addPlayer($playerDto);
        if (!$resultCreatePlayer->success) {
            DB::rollback();
            return response()->json(['error' => ['mensaje' => ["Error al crear el participante"]]], 400);
        }

        $urlLogin = \config('app_urls.confirm_url');
        $details = [
            'nombre_completo' => $nombreCompleto,
            'password' => $randomPassword,
            'url' => $urlLogin,
            'email' => $resultCreateUser->user->email
        ];
        $sendEmailInvitacion = $this->emailService->SendEmailInvitacionParticipante($details);
        if (!$sendEmailInvitacion) {
            DB::rollback();
            return response()->json(['error' => ['mensaje' => ["Error al notificar al participante"]]], 400);
        }
        DB::commit();
        return response()->json([
            'message' => 'El participante se creó correctamente',
            'data' => $resultCreatePlayer->player
        ], Response::HTTP_OK);
    }

    /**
     * This PHP function updates a player's information based on the provided request data and returns
     * a JSON response with the updated player data or an error message.
     * 
     * @param UpdatePlayerRequest request The `update` function in the code snippet is responsible for
     * updating player information based on the data provided in the `UpdatePlayerRequest` object and
     * the player's ID.
     * @param id The `id` parameter in the `update` function is typically used to identify the specific
     * player that needs to be updated. It is likely an identifier, such as a unique numerical ID, that
     * corresponds to the player in the database. This ID helps the function locate the correct player
     * record to update with
     * 
     * @return The `update` function is returning a JSON response. If the validation fails, it returns
     * an error response with the validation messages. If the player update is successful, it returns a
     * success message along with the updated player data.
     */
    public function update(UpdatePlayerRequest $request, $id)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $nombreTienda = $request->nombre_tienda ?? "";
        $direccion = $request->direccion ?? "";
        $nombreCompleto = $request->nombre_completo ?? "";
        $localidad = $request->localidad ?? "";
        $estado = $request->estado ?? "";
        $ciudad = $request->ciudad ?? "";
        $ruta = $request->ruta ?? "";
        $fileId = $request->file_id ?? "";
        $playerDto = new PlayerDto($nombreTienda, $direccion, $nombreCompleto, $localidad, $estado, $ciudad, $ruta, $fileId, "", "");
        $resultUpdatePlayer = $this->playerService->updatePlayer($playerDto, $id);
        if (!$resultUpdatePlayer->success) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el participante"]]], 400);
        }
        return response()->json([
            'message' => 'El participante se actualizó correctamente',
            'data' => $resultUpdatePlayer->player
        ], Response::HTTP_OK);
    }

    /**
     * The function `destroy` deletes a participant along with associated data and user access,
     * handling potential errors and transactions in PHP.
     * 
     * @param id The `destroy` function you provided seems to be a method in a Laravel controller that
     * is responsible for deleting a participant along with associated data. The `id` parameter in this
     * function is the unique identifier of the participant that needs to be deleted.
     * 
     * @return a JSON response with a success message 'Se eliminó el participante correctamente' and a
     * status code of 200 (HTTP OK) if the participant is successfully deleted. If there are any errors
     * during the deletion process, it will return a JSON response with an error message and a status
     * code of 400.
     */
    public function destroy($id)
    {
        DB::beginTransaction();
        $participanteFind = Participante::find($id);

        if (!$participanteFind)
            return response()->json(['error' => ['mensaje' => ["El participante no existe"]]], 400);

        $existRelacionesParticipante = $participanteFind->secureDelete("retos");
        if ($existRelacionesParticipante){
            DB::rollback();
            return response()->json(['error' => ['mensaje' => ["La participante ha sido asignado a los retos"]]], 400);
        }

        $resultDeletePlayer = $this->playerService->removePlayerById($id);
        if (!$resultDeletePlayer) {
            DB::rollback();
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el participante"]]], 400);
        }

        $resultUser = $this->authService->removeUserById($participanteFind->user_id);
        if (!$resultUser) {
            DB::rollback();
            return response()->json(['error' => ['mensaje' => ["Error al eliminar los accesos del participante"]]], 400);
        }
        DB::commit();
        return response()->json([
            'message' => 'Se eliminó el participante correctamente'
        ], Response::HTTP_OK);
    }
}
