<?php

namespace App\Http\Controllers\V1\Producto;

use App\Dto\ProductDto;
use App\Http\Controllers\Controller;
use App\Http\Requests\ProductRequest;
use App\Http\Requests\ProductUpdateRequest;
use App\Models\Producto;
use App\Services\ProductService;
use Symfony\Component\HttpFoundation\Response;

class ProductoController extends Controller
{
    protected $productService;

    public function __construct(ProductService $productService)
    {
        $this->productService = $productService;
    }

    /**
     * The index function retrieves all products and returns them as JSON, handling errors if the
     * retrieval is unsuccessful.
     * 
     * @return the data from the getAllProducts method if the operation is successful. If there is an
     * error, it will return a JSON response with an error message "Error al consultar los productos"
     * and a status code of 400.
     */
    public function index()
    {
        $result = $this->productService->getAllProducts();
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los productos"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function `show` retrieves a product by its ID and returns the product data or an error
     * message if the product retrieval fails.
     * 
     * @param id The `show` function takes an `` parameter, which is used to retrieve a product from
     * the ProductService by calling the `getProductById` method with the provided ``. If the
     * retrieval is successful (`->success` is true), it returns the product data. Otherwise, it
     * 
     * @return the data of a product with the given ID if the retrieval was successful. If there was an
     * error during the retrieval process, it returns a JSON response with an error message indicating
     * "Error al consultar el producto" and a status code of 400.
     */
    public function show($id)
    {
        $result = $this->productService->getProductById($id);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el producto"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function `store` processes a ProductRequest, validates it, creates a ProductDto, adds the
     * product using a service, and returns a JSON response with the result.
     * 
     * @param ProductRequest request The `store` function in the code snippet is responsible for
     * storing a new product based on the data provided in the `ProductRequest`. Here is an explanation
     * of the parameters used in the function:
     * 
     * @return The `store` function is returning a JSON response. If the validation fails, it returns
     * an error message with the validation messages. If the product creation is successful, it returns
     * a success message along with the created product data.
     */
    public function store(ProductRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $productDto = new ProductDto();
        $productDto->nombre = $request->nombre;
        $productDto->descripcion = $request->descripcion;
        $productDto->fileId = $request->file_id;
        $productDto->userId = $request->user_id;

        $resultProduct = $this->productService->addProduct($productDto);
        if (!$resultProduct->success) {
            return response()->json(['error' => ['mensaje' => ["Error al crear el producto"]]], 400);
        }
        return response()->json([
            'message' => 'El producto se creó correctamente',
            'data' => $resultProduct->product
        ], Response::HTTP_OK);
    }

    /**
     * This PHP function updates a product based on the data provided in the request and returns a JSON
     * response with the result.
     * 
     * @param ProductUpdateRequest request The `update` function in the code snippet is responsible for
     * updating a product based on the data provided in the `ProductUpdateRequest` object. Here's a
     * breakdown of the parameters used in the function:
     * @param id The `id` parameter in the `update` function is typically used to identify the specific
     * product that needs to be updated. It is usually an identifier (such as a numeric ID or a unique
     * key) that helps to locate the product in the database or storage system. This `id` is crucial
     * 
     * @return The `update` function is returning a JSON response. If the validation fails, it returns
     * an error message with the validation messages. If the product update is successful, it returns a
     * success message along with the updated product data.
     */
    public function update(ProductUpdateRequest $request, $id)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $productDto = new ProductDto();
        $productDto->id = $id;
        $productDto->nombre = $request->nombre;
        $productDto->descripcion = $request->descripcion;
        $productDto->fileId = $request->file_id;
        $productDto->estatus = $request->estatus;
        
        $resultProduct = $this->productService->updateProduct($productDto, $id);
        if (!$resultProduct->success) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el producto"]]], 400);
        }
        return response()->json([
            'message' => 'El producto se actualizo correctamente',
            'data' => $resultProduct->product
        ], Response::HTTP_OK);
    }

    /**
     * The function `destroy` deletes a product by its ID and returns appropriate JSON responses based
     * on the outcome.
     * 
     * @param id The `destroy` function in the code snippet is used to delete a product by its ID. The
     * function first tries to find the product with the given ID using `Producto::find()`. If the
     * product is not found, it returns a JSON response with an error message stating that the product
     * 
     * @return The `destroy` function is returning a JSON response. If the product with the specified
     * ID is not found, it returns an error message indicating that the product does not exist. If
     * there is an error while trying to delete the product, it returns an error message stating that
     * there was an issue with deleting the product. If the product is successfully deleted, it returns
     * a success message indicating that the product was
     */
    public function destroy($id)
    {
        $productoDelete = Producto::find($id);
        if (!$productoDelete)
            return response()->json(['error' => ['mensaje' => ["El producto no existe"]]], 400);
        $result = $this->productService->removeProductById($id);

        if (!$result) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el producto"]]], 400);
        }
        return response()->json([
            'message' => 'Se eliminó el producto correctamente'
        ], Response::HTTP_OK);
    }
}
