<?php

namespace App\Repositories;

use App\Dto\RetoAdvancePlayers;
use App\Dto\RetoDto;
use App\Interfaces\RetoInterface;
use App\Models\EvidenciaReto;
use App\Models\NivelReto;
use App\Models\Reto;
use App\Models\RetoParticipantes;
use App\Models\RetoProducto;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class RetoRepository implements RetoInterface
{
    public function allRetos()
    {
        $retos = Reto::with([
            "participantes",
            "productos",
            'nivel_retos.insignias.files', 'files',
            'evidencias_retos' => function ($query) {
                $query->orderBy('fecha_captura', 'desc');
            }
        ])
            ->get();
        return $retos;
    }

    public function findRetosByTypePlayer(string $tipoReto)
    {
        $retos = Reto::with([
            "participantes",
            "productos",
            'nivel_retos.insignias.files', 'files',
            'evidencias_retos' => function ($query) {
                $query->orderBy('fecha_captura', 'desc');
            }
        ])
            ->where("tipo_participante", $tipoReto)
            ->get();
        return $retos;
    }

    public function detailReto(string $idReto)
    {
        $retoQuery = Reto::with([
            "participantes",
            "productos",
            'nivel_retos.insignias.files'  => function ($query) {
                $query->orderBy('created_at', 'asc');
            }, 'files'
        ])
            ->where('id', $idReto)->first();
        return $retoQuery;
    }

    public function findRetosVigentesByPlayer(string $idPlayer)
    {
        $currentDateTime = Carbon::now();
        $dateNow = $currentDateTime->format('Y-m-d');
        $retosQuery = Reto::with(['nivel_retos' => function ($query) use ($dateNow) {
            $query->where('nivel_retos.fecha_inicio', '<=', $dateNow);
            $query->where('nivel_retos.fecha_fin', '>=', $dateNow);
        }])
            ->whereHas('participantes', function ($query) use ($idPlayer) {
                $query->where('participantes.id', $idPlayer);
            })
            ->get();
        return $retosQuery;
    }

    public function filterRetosWithProductAndPlayer(string $idPlayer, $listProductsId)
    {
        $currentDateTime = Carbon::now();
        $dateNow = $currentDateTime->format('Y-m-d');

        $retosQueryFilter = Reto::with(['nivel_retos' => function ($query) use ($dateNow) {
            $query->where('nivel_retos.fecha_inicio', '<=', $dateNow);
            $query->where('nivel_retos.fecha_fin', '>=', $dateNow);
        }])
            ->whereHas('participantes', function ($query) use ($idPlayer) {
                $query->where('participantes.id', $idPlayer);
            })
            ->whereHas('productos', function ($query) use ($listProductsId) {
                $query->whereIn('productos.id', $listProductsId);
            })
            ->get();
        return $retosQueryFilter;
    }

    public function findRetosByPlayer(string $idPlayer)
    {
        $retosQuery = Reto::select(
            'retos.id',
            'retos.nombre',
            'retos.descripcion',
            'retos.tipo_reto',
            'retos.file_id'
        )
            ->with(['nivel_retos' => function ($query) {
                $query->orderBy('meta', 'asc');
            }, 'files'])
            ->whereHas('participantes', function ($query) use ($idPlayer) {
                $query->where('participantes.id', $idPlayer);
            })
            ->addSelect([
                DB::raw('(SELECT MIN(nivel_retos.fecha_inicio) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_inicio'),
                DB::raw('(SELECT MIN(nivel_retos.fecha_fin) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_fin'),
                DB::raw('(SELECT MAX(nivel_retos.meta) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as meta_total')
            ])
            ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.cantidad) FROM evidencia_retos
            LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
            LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
            WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
            AND evidencias.participante_id = ?) as cantidad_total_acumulada'), [$idPlayer])
            ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.monto) FROM evidencia_retos
            LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
            LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
            WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
            AND evidencias.participante_id = ?) as monto_total_acumulada'), [$idPlayer])
            ->groupBy('retos.id', 'retos.nombre', 'retos.descripcion', 'retos.tipo_reto', 'retos.file_id')
            ->get()
            ->map(function ($reto) {
                $avance = 0;
                $meta_total = $reto->meta_total;
                if ($reto->tipo_reto == 'cantidad') {
                    $cantidadTotalAcumulada = ($reto->cantidad_total_acumulada) ? $reto->cantidad_total_acumulada : 0;
                    $avance = $cantidadTotalAcumulada / $meta_total * 100;
                } else {
                    $floatMontoTotalAcumulado = ($reto->monto_total_acumulada) ? floatval($reto->monto_total_acumulada) : 0;
                    $avance = $floatMontoTotalAcumulado  / $meta_total * 100;
                }
                $reto->avance = ($avance > 100) ? 100 : $avance;
                return $reto;
            });
        return $retosQuery;
    }

    public function findAvanceRetosWithRol(string $rol)
    {
        $retos = Reto::select(
            'retos.id',
            'retos.nombre',
            'retos.descripcion',
            'retos.tipo_reto',
            'retos.file_id'
        )
            ->with(['files'])
            ->whereHas('participantes.usuario.roles', function ($query) use ($rol) {
                $query->where('name', $rol);
            })
            ->addSelect([
                DB::raw('(SELECT COUNT(*) FROM reto_participantes WHERE reto_participantes.reto_id = retos.id) as num_participantes'),
                DB::raw('(SELECT SUM(nivel_retos.meta) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as meta_total'),
                DB::raw('(SELECT SUM(nivel_retos.meta) FROM nivel_retos
                JOIN historial_recompensas ON nivel_retos.id = historial_recompensas.nivel_id 
                WHERE nivel_retos.reto_id = retos.id) as meta_participantes'),
                DB::raw('(SELECT MIN(nivel_retos.fecha_fin) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_fin'),
            ])
            ->get()
            ->map(function ($reto) {
                $avance = 0;
                $tipoReto = $reto->tipo_reto;
                $numParticipanteInRetos = intval($reto->num_participantes);
                $metaTotal = intval($reto->meta_total);
                //cantidad de dinero o cantidad acumulado de todos los participantes, tomo en cuenta el historial de recompensas
                //por cada registro que se genera ahi indica que se cumplio un nivel por participante.
                $metaAcumuladaInRetos = intval($reto->meta_participantes);
                $metaRetoTotalAllPlayers = $numParticipanteInRetos * $metaTotal;

                $avance = $metaAcumuladaInRetos / $metaRetoTotalAllPlayers * 100;
                $retoAdvance = new RetoAdvancePlayers();
                $retoAdvance->id = $reto->id;
                $retoAdvance->nombre = $reto->nombre;
                $retoAdvance->descripcion = $reto->descripcion;
                $retoAdvance->numParticipantes = $numParticipanteInRetos;
                $retoAdvance->tipoReto = $tipoReto;
                $retoAdvance->avance = $avance;
                $retoAdvance->files = $reto->files;
                $retoAdvance->fechaFin = $reto->fecha_fin;
                return $retoAdvance;
            });
        return $retos;
    }

    public function findDetailRetoByPlayer(string $idReto, string $idPlayer)
    {
        $reto = Reto::select(
            'retos.id',
            'retos.nombre',
            'retos.descripcion',
            'retos.tipo_reto',
        )
            ->with(['nivel_retos' => function ($query) {
                $query->orderBy('meta', 'asc');
            }])
            ->with(['nivel_retos.insignias.files', 'nivel_retos.insignias.recompensa.files'])
            ->where('retos.id', $idReto)
            ->whereHas('participantes', function ($query) use ($idPlayer) {
                $query->where('participantes.id', $idPlayer);
            })
            ->addSelect([
                DB::raw('(SELECT MIN(nivel_retos.fecha_inicio) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_inicio'),
                DB::raw('(SELECT MIN(nivel_retos.fecha_fin) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_fin'),
                DB::raw('(SELECT MAX(nivel_retos.meta) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as meta_total')
            ])
            ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.cantidad) FROM evidencia_retos
            LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
            LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
            WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
            AND evidencias.participante_id = ?) as cantidad_total_acumulada'), [$idPlayer])
            ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.monto) FROM evidencia_retos
            LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
            LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
            WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
            AND evidencias.participante_id = ?) as monto_total_acumulada'), [$idPlayer])
            ->selectRaw(DB::raw('(SELECT COUNT(*) FROM evidencias
            WHERE evidencias.estatus = "aprobado"
            AND evidencias.participante_id = ?) as total_ev_apro'), [$idPlayer])
            ->selectRaw(DB::raw('(SELECT COUNT(*) FROM evidencias
            WHERE evidencias.estatus = "evaluar"
            AND evidencias.participante_id = ?) as total_ev_pend'), [$idPlayer])
            ->selectRaw(DB::raw('(SELECT COUNT(*) FROM evidencias
            WHERE evidencias.estatus = "rechazada"
            AND evidencias.participante_id = ?) as total_ev_recha'), [$idPlayer])
            ->groupBy('retos.id', 'retos.nombre', 'retos.descripcion', 'retos.tipo_reto')
            ->first();

        if ($reto) {
            $avance = 0;
            $meta_total = $reto->meta_total;
            if ($reto->tipo_reto == 'cantidad') {
                $cantidadTotalAcumulada = ($reto->cantidad_total_acumulada) ? $reto->cantidad_total_acumulada : 0;
                $avance = $cantidadTotalAcumulada / $meta_total * 100;
            } else {
                $floatMontoTotalAcumulado = ($reto->monto_total_acumulada) ? floatval($reto->monto_total_acumulada) : 0;
                $avance = $floatMontoTotalAcumulado  / $meta_total * 100;
            }
            $reto->avance = ($avance > 100) ? 100 : $avance;
        }
        return $reto;
    }

    public function filterRetosWhereNotProducts(string $idReto, $listProdutsId)
    {
        $queryRetosProducts = RetoProducto::where('reto_id', $idReto)
            ->whereNotIn('producto_id', $listProdutsId)->get();
        return $queryRetosProducts;
    }

    public function filterRetosWhereNotPlayers(string $idReto, $listPlayers)
    {
        $queryFilter = RetoParticipantes::where('reto_id', $idReto)
            ->whereNotIn('participante_id', $listPlayers)->get();
        return $queryFilter;
    }

    public function existRetoWithProduct(string $idReto, string $idProducto)
    {
        $exitsRetoProducto = RetoProducto::where('reto_id', '=', $idReto)
            ->where('producto_id', '=', $idProducto)->exists();
        return $exitsRetoProducto;
    }

    public function existRetoWithPlayer(string $idReto, string $participanteId)
    {
        $exitsRetoPlayer = RetoParticipantes::where('reto_id', '=', $idReto)
            ->where('participante_id', '=', $participanteId)->exists();
        return $exitsRetoPlayer;
    }

    public function existRetoWithEvidencia(string $idReto, string $evidenciaId)
    {
        $exitsEvidenciaReto = EvidenciaReto::where('reto_id', '=', $idReto)
            ->where('evidencia_id', '=', $evidenciaId)->exists();
        return $exitsEvidenciaReto;
    }


    public function createReto(RetoDto $retoDto)
    {
        $retoCreate = Reto::create([
            "nombre" => $retoDto->nombre,
            "descripcion" => $retoDto->descripcion,
            "tipo_reto" => $retoDto->tipoReto,
            "tipo_participante" => $retoDto->tipoParticipante,
            "user_id" => $retoDto->userId,
            'file_id' => $retoDto->fileId
        ]);
        return $retoCreate;
    }

    public function createRetoRelationProducto(string $idReto, string $idProducto)
    {
        $retoRelation = RetoProducto::create([
            "reto_id" => $idReto,
            "producto_id" => $idProducto,
        ]);
        return $retoRelation;
    }

    public function createRetoRelationPlayer(string $idReto, string $idPlayer)
    {
        $retoRelation = RetoParticipantes::create([
            "reto_id" => $idReto,
            "participante_id" => $idPlayer,
        ]);
        return $retoRelation;
    }

    public function updateReto(RetoDto $retoDto, string $idReto)
    {
        $retoFind = Reto::findOrFail($idReto);
        $retoFind->update([
            "nombre" => $retoDto->nombre,
            "descripcion" => $retoDto->descripcion,
            "tipo_reto" => $retoDto->tipoReto,
            'file_id' => $retoDto->fileId
        ]);
        return $retoFind;
    }

    public function deleteReto(string $idReto)
    {
        $retoFind = Reto::findOrFail($idReto);
        $retoFind->delete();
    }

    public function deleteRelationRetoProduct(string $idRetoProduct)
    {
        $retoProduct = RetoProducto::findOrFail($idRetoProduct);
        $retoProduct->delete();
    }

    public function deleteRelationRetoPlayer(string $idRetoPlayer)
    {
        $retoPlayer = RetoParticipantes::findOrFail($idRetoPlayer);
        $retoPlayer->delete();
    }
}
