<?php

namespace App\Services;

use App\Dto\Responses\ResponseDataDto;
use App\Dto\Responses\ResponseRetoDto;
use App\Dto\RetoDto;
use App\Repositories\RetoRepository;
use Illuminate\Support\Facades\Log;

class RetoService
{
    protected $retoRepository;

    public function __construct(RetoRepository $retoRepository)
    {
        $this->retoRepository = $retoRepository;
    }

    public function getRetosAll()
    {
        try {
            $allQuery = $this->retoRepository->allRetos();
            return new ResponseDataDto(true, $allQuery);
        } catch (\Exception $e) {
            Log::error('Error al consultar los retos: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRetosByTypePlayer(string $typeReto)
    {
        try {
            $queryFilter = $this->retoRepository->findRetosByTypePlayer($typeReto);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar los retos por tipo de participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRetosVigentesByPlayer(string $idPlayer)
    {
        try {
            $queryFilter = $this->retoRepository->findRetosVigentesByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar los retos vigentes del participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRetosWithProductsByPlayer(string $idPlayer, $listProductsId)
    {
        try {
            $queryFilter = $this->retoRepository->filterRetosWithProductAndPlayer($idPlayer, $listProductsId);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar los retos que estan relacionados a los productos y participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRetosByPlayer(string $idPlayer)
    {
        try {
            $queryFilter = $this->retoRepository->findRetosByPlayer($idPlayer);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar los retos asignados al participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRetoDetailByPlayer(string $idReto, string $idPlayer)
    {
        try {
            $queryFilter = $this->retoRepository->findDetailRetoByPlayer($idReto, $idPlayer);
            return new ResponseDataDto(true, $queryFilter);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle del reto asignado al participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, []);
        }
    }

    public function getRetoById(string $idReto)
    {
        try {
            $queryDetail = $this->retoRepository->detailReto($idReto);
            return new ResponseDataDto(true, $queryDetail);
        } catch (\Exception $e) {
            Log::error('Error al consultar el detalle del reto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function addReto(RetoDto $retoDto)
    {
        try {
            $newItem = $this->retoRepository->createReto($retoDto);
            $retoDto->id = $newItem->id;
            return new ResponseRetoDto(true, $retoDto);
        } catch (\Exception $e) {
            Log::error('Error al crear el reto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseRetoDto(false, null);
        }
    }

    public function addRetoToProduct(string $idReto, string $idProduct)
    {
        try {
            $newItem = $this->retoRepository->createRetoRelationProducto($idReto, $idProduct);
            return new ResponseDataDto(true, $newItem);
        } catch (\Exception $e) {
            Log::error('Error al asignar el reto al producto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function addRetoToPlayer(string $idReto, string $idPlayer)
    {
        try {
            $newItem = $this->retoRepository->createRetoRelationPlayer($idReto, $idPlayer);
            return new ResponseDataDto(true, $newItem);
        } catch (\Exception $e) {
            Log::error('Error al asignar el reto al participante: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }

    public function updateReto(RetoDto $retoDto, string $retoId)
    {
        try {
            $this->retoRepository->updateReto($retoDto, $retoId);
            return new ResponseRetoDto(true, $retoDto);
        } catch (\Exception $e) {
            Log::error('Error al actualizar el reto: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseRetoDto(false, null);
        }
    }

    public function removeRetoById(string $retoId)
    {
        try {
            $this->retoRepository->deleteReto($retoId);
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar el reto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function removeProductByRetoRelation(string $idReto, $productos)
    {
        try {
            $listProductsId = array_column($productos, 'producto_id');
            $productRetoToDelete = $this->retoRepository->filterRetosWhereNotProducts($idReto, $listProductsId);
            $productRetoRelationArrayDelete = $productRetoToDelete->toArray();
            if (!empty($productRetoRelationArrayDelete)) {
                foreach ($productRetoRelationArrayDelete as $valor) {
                    $idRetoProduct = $valor['id'];
                    $this->retoRepository->deleteRelationRetoProduct($idRetoProduct);
                }
            }
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar la relación del reto con el producto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function removePlayersByRetoRelation(string $idReto, $listPlayers)
    {
        try {
            $getPlayersDelete = $this->retoRepository->filterRetosWhereNotPlayers($idReto, $listPlayers);
            $partiRelationArrayDelete = $getPlayersDelete->toArray();
            if (!empty($partiRelationArrayDelete)) {
                foreach ($partiRelationArrayDelete as $valor) {
                    $idRetoPlayer = $valor['id'];
                    $this->retoRepository->deleteRelationRetoPlayer($idRetoPlayer);
                }
            }
            return true;
        } catch (\Exception $e) {
            Log::error('Error al eliminar la relación del reto con el participante: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function existAssingProductWithReto(string $idReto, string $idProduct)
    {
        try {
            $resultExist = $this->retoRepository->existRetoWithProduct($idReto, $idProduct);
            return $resultExist;
        } catch (\Exception $e) {
            Log::error('Error al consultar si el producto ya esta asignado al reto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function existAssingPlayerWithReto(string $idReto, string $idPlayer)
    {
        try {
            $resultExist = $this->retoRepository->existRetoWithPlayer($idReto, $idPlayer);
            return $resultExist;
        } catch (\Exception $e) {
            Log::error('Error al consultar si el participante ya esta asignado al reto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function existAssingEvidenciaWithReto(string $idReto, string $idEvidencia)
    {
        try {
            $resultExist = $this->retoRepository->existRetoWithEvidencia($idReto, $idEvidencia);
            return $resultExist;
        } catch (\Exception $e) {
            Log::error('Error al consultar si la evidencia esta asignada al reto: ' . $e->getMessage(), ['exception' => $e]);
            return false;
        }
    }

    public function getAdvanceRetosByRol($rol)
    {
        try {
            $queryDetail = $this->retoRepository->findAvanceRetosWithRol($rol);
            return new ResponseDataDto(true, $queryDetail);
        } catch (\Exception $e) {
            Log::error('Error al consultar los avances de los retos: ' . $e->getMessage(), ['exception' => $e]);
            return new ResponseDataDto(false, null);
        }
    }
}
